'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _fs = require('fs');

var _path = require('path');

var _lodash = require('lodash.isfunction');

var _lodash2 = _interopRequireDefault(_lodash);

var _roc = require('roc');

var _small = require('roc/log/default/small');

var _small2 = _interopRequireDefault(_small);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) arr2[i] = arr[i]; return arr2; } else { return Array.from(arr); } }

exports.default = {
    actions: [{
        hook: 'babel-config',
        description: 'Base Babel configuration',
        action: () => () => babelConfig => _extends({}, babelConfig, {
            // We disable babelrc here since it often results in strange behaviors
            babelrc: false,
            presets: [],
            plugins: [],
            env: {}
        }),
        post: _ref => {
            var _ref$context = _ref.context;
            let userConfig = _ref$context.config;
            let directory = _ref$context.directory;
            let packageJSON = _ref$context.packageJSON;
            return target => {
                let userBabelConfig;

                /**
                The priority when Roc loads Babel configuration.
                 1. "babel" inside roc.config.js
                2. .babelrc
                3. "babel" inside package.json
                */
                if (userConfig.babel) {
                    if ((0, _roc.fileExists)('.babelrc', directory)) {
                        _small2.default.warn('You have defined a Babel configuration in the roc.config.js file that will be used ' + 'over the existing .babelrc file.');
                    } else if (packageJSON.babel) {
                        _small2.default.warn('You have defined a Babel configuration in the roc.config.js file that will be used ' + 'over the configuration inside package.json.');
                    }

                    userBabelConfig = (0, _lodash2.default)(userConfig.babel) ? userConfig.babel(target) : userConfig.babel;
                } else if ((0, _roc.fileExists)('.babelrc', directory)) {
                    userBabelConfig = JSON.parse((0, _fs.readFileSync)((0, _path.join)(directory, '.babelrc'), 'utf-8'));
                } else if (packageJSON.babel) {
                    userBabelConfig = packageJSON.babel;
                }

                if (userBabelConfig) {
                    return babelConfig => {
                        const newBabelConfig = (0, _roc.merge)(babelConfig, userBabelConfig);
                        newBabelConfig.plugins = [].concat(_toConsumableArray(userBabelConfig.plugins || []), _toConsumableArray(babelConfig.plugins || []));

                        // We need to flip the order here because of the way that Babel processes presets
                        newBabelConfig.presets = [].concat(_toConsumableArray(babelConfig.presets || []), _toConsumableArray(userBabelConfig.presets || []));

                        // Merge env configuration with special consideration for plugins & presets
                        Object.keys(newBabelConfig.env).forEach(env => {
                            const babelConfigEnv = babelConfig.env[env] || {};
                            const userBabelConfigEnv = userBabelConfig.env[env] || {};

                            const envPresets = [].concat(_toConsumableArray(babelConfigEnv.presets || []), _toConsumableArray(userBabelConfigEnv.presets || []));

                            newBabelConfig.env[env].presets = envPresets;

                            const envPlugins = [].concat(_toConsumableArray(userBabelConfigEnv.plugins || []), _toConsumableArray(babelConfigEnv.plugins || []));

                            newBabelConfig.env[env].plugins = envPlugins;
                        });

                        return newBabelConfig;
                    };
                }

                return undefined;
            };
        }
    }]
};
//# sourceMappingURL=index.js.map